package com.zanxiang.game.back.serve.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.game.back.serve.dao.mapper.GameOceanengineUserLogMapper;
import com.zanxiang.game.back.serve.oceanengine.MiniGameCallback;
import com.zanxiang.game.back.serve.oceanengine.OceanengineCallbackException;
import com.zanxiang.game.back.serve.pojo.entity.GameOceanengineBackLog;
import com.zanxiang.game.back.serve.pojo.entity.GameOceanengineUserLog;
import com.zanxiang.game.back.serve.pojo.enums.BackStatusEnum;
import com.zanxiang.game.back.serve.service.IGameOceanengineBackLogService;
import com.zanxiang.game.back.serve.service.IGameOceanengineUserLogService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;

@Slf4j
@Service
public class GameOceanengineUserLogServiceImpl extends ServiceImpl<GameOceanengineUserLogMapper, GameOceanengineUserLog>
        implements IGameOceanengineUserLogService {

    @Autowired
    private MiniGameCallback miniGameCallback;
    @Autowired
    private IGameOceanengineBackLogService gameOceanengineBackLogService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean callback(GameOceanengineUserLog userLog, boolean mustBack) {
        BackStatusEnum backStatus = doCallback(userLog, mustBack);
        return update(new LambdaUpdateWrapper<GameOceanengineUserLog>()
                .set(GameOceanengineUserLog::getBackStatus, backStatus.getBackStatus())
                .eq(GameOceanengineUserLog::getId, userLog.getId())
        );
    }

    private BackStatusEnum doCallback(GameOceanengineUserLog userLog, boolean mustBack) {
        // 是否已经回传过
        boolean isBack = mustBack ? false : gameOceanengineBackLogService.count(new LambdaQueryWrapper<GameOceanengineBackLog>()
                .eq(GameOceanengineBackLog::getGameId, userLog.getGameId())
                .eq(GameOceanengineBackLog::getAppId, userLog.getAppId())
                .eq(GameOceanengineBackLog::getOpenId, userLog.getOpenId())
                .eq(GameOceanengineBackLog::getAccountId, userLog.getAccountId())
                .eq(GameOceanengineBackLog::getBackStatus, BackStatusEnum.SUCCESS)
        ) > 0;
        if (isBack) {
            return BackStatusEnum.NO;
        }
        MiniGameCallback.MiniGameCallbackRequest request = MiniGameCallback.MiniGameCallbackRequest.builder()
                .clue_token(userLog.getClueToken())
                .open_id(userLog.getOpenId())
                .union_id(userLog.getUnionId())
                .event_type(String.valueOf(MiniGameCallback.EventType.ACTIVE.getEventType()))
                .build();
        BackStatusEnum backStatus = BackStatusEnum.SUCCESS;
        String errMsg = null;
        try {
            miniGameCallback.doCallback(userLog.getAccountReportUrl(), userLog.getAccountReportToken(), request);
        } catch (OceanengineCallbackException e) {
            backStatus = BackStatusEnum.FAILED;
            errMsg = e.getMsg();
        } catch (Exception e) {
            backStatus = BackStatusEnum.FAILED;
            errMsg = "系统异常：" + e.getMessage();
        }
        gameOceanengineBackLogService.save(GameOceanengineBackLog.builder()
                .appId(userLog.getAppId())
                .openId(userLog.getOpenId())
                .unionId(userLog.getUnionId())
                .gameId(userLog.getGameId())
                .accountId(userLog.getAccountId())
                .clueToken(userLog.getClueToken())
                .eventType(request.getEvent_type())
                .eventLogId(userLog.getId())
                .createTime(LocalDateTime.now())
                .backStatus(backStatus.getBackStatus())
                .errMsg(errMsg)
                .build());
        return backStatus;
    }
}
